import json

from airflow.plugins_manager import AirflowPlugin
from airflow_docker.utils import get_config
from airflow_docker.views import template_folder
from flask import Blueprint
from flask_admin import BaseView, expose
from flask_appbuilder import BaseView as AppBuilderBaseView
from pygments import highlight, lexers
from pygments.formatters import HtmlFormatter


def render_config(self):
    config = get_config()
    data = json.dumps(config, indent=4, sort_keys=True)
    return highlight(data, lexers.JsonLexer(), HtmlFormatter(linenos=True))


class ConfigView(BaseView):
    @expose("/")
    def config(self):
        rendered_config = render_config(self)
        return self.render("config.html", data=rendered_config)


class AppBuilderConfigView(AppBuilderBaseView):
    template_folder = template_folder

    @expose("/")
    def list(self):
        rendered_config = render_config(self)
        return self.render_template(
            "config.html", data=rendered_config, appbuilder=True
        )


view = ConfigView(category="Airflow Docker", name="Config")
blueprint = Blueprint("config", __name__, template_folder=template_folder)

appbuilder_view = AppBuilderConfigView()
appbuilder_package = {
    "name": "Config",
    "category": "Airflow Docker",
    "view": appbuilder_view,
}



from airflow.plugins_manager import AirflowPlugin
from airflow_docker.operator import (
    BranchOperator,
    Operator,
    Sensor,
    ShortCircuitOperator,
)
from airflow_docker.views import config, extensions


class AirflowDockerPlugin(AirflowPlugin):
    name = "airflow_docker"

    operators = [BranchOperator, Operator, ShortCircuitOperator]
    sensors = [Sensor]
    admin_views = [config.view, extensions.view]
    flask_blueprints = [config.blueprint, extensions.blueprint]

    appbuilder_views = [config.appbuilder_package, extensions.appbuilder_package]



import os
import textwrap

import boto3

MAX_ROLE_DURATION_SECONDS = 60 * 60 * 12  # 12 hours in seconds
MAX_ROLE_SESSION_NAME_LENGTH = 64
ROLE_SESSION_NAME_TEMPLATE = "{dag_run_id}__{task_instance_try}__{task_id}"

credential_key_map = {
    "aws_access_key_id": "AccessKeyId",
    "aws_secret_access_key": "SecretAccessKey",
    "aws_session_token": "SessionToken",
}


def generate_role_credentials(
    role_arn, session_name, role_session_duration=MAX_ROLE_DURATION_SECONDS
):
    client = boto3.client("sts")

    response = client.assume_role(
        RoleArn=role_arn,
        RoleSessionName=session_name,
        DurationSeconds=role_session_duration,
    )

    return response


def format_credentials_data(raw_credentials):
    credentials = {
        key: raw_credentials["Credentials"][mapped_key]
        for key, mapped_key in credential_key_map.items()
    }
    return credentials


def aws_credentials_file_format(
    aws_access_key_id, aws_secret_access_key, aws_session_token, profile="default"
):
    credentials = textwrap.dedent(
        f"""\
        [{profile}]
        aws_access_key_id={aws_access_key_id}
        aws_secret_access_key={aws_secret_access_key}
        aws_session_token={aws_session_token}
        """
    )
    return credentials


def generate_role_session_name(context):
    return ROLE_SESSION_NAME_TEMPLATE.format(
        dag_run_id=context["dag_run"].id,
        task_instance_try=context["task_instance"].try_number,
        task_id=context["task_instance"].task_id,
    )[:MAX_ROLE_SESSION_NAME_LENGTH]


def get_credentials(context, role_arn, role_session_duration=MAX_ROLE_DURATION_SECONDS):
    raw_credentials = generate_role_credentials(
        role_arn=role_arn,
        session_name=generate_role_session_name(context),
        role_session_duration=role_session_duration,
    )
    return raw_credentials


def write_credentials(credentials, credentials_path):
    credentials_file_contents = aws_credentials_file_format(**credentials)
    os.makedirs(os.path.dirname(credentials_path), exist_ok=True)
    with open(credentials_path, "wb") as f:
        f.write(credentials_file_contents.encode("utf-8"))


def find_role_session_duration(role_arn):
    iam = boto3.client("iam")
    response = iam.get_role(RoleName=parse_role_name_from_arn(role_arn))
    return response["Role"]["MaxSessionDuration"]


def log_credentials(operator, raw_credentials):
    access_key_id = raw_credentials["Credentials"]["AccessKeyId"]
    expiration = raw_credentials["Credentials"]["Expiration"]
    user_arn = raw_credentials["AssumedRoleUser"]["AssumedRoleId"]

    operator.log.info("Assumed Role:")
    operator.log.info("  Access Key ID = {}".format(access_key_id))
    operator.log.info("  Expiration    = {}".format(expiration.isoformat()))
    operator.log.info("  Arn           = {}".format(user_arn))


def parse_role_name_from_arn(role_arn):
    role_name_start = role_arn.index("role") + 5
    return role_arn[role_name_start:]


class AWSRoleAssumptionExtension:
    """Assume a role for your task.

    Session will have a session name of the following template:

        {dag_run_id}__{task_instance_try}__{task_id}

    Operator Keyword Arguments:
        role_arn: The role arn you want to assume
        role_session_duration: The number of seconds to assume the role for.
            Min: 900, Max: 43200 secs
            Defaults to Max of 43200 seconds (12 hours).
    """

    kwargs = {"role_arn", "role_session_duration"}

    @classmethod
    def post_prepare_environment(
        cls, operator, config, context, host_tmp_dir, session=None
    ):
        if "role_arn" in operator.extra_kwargs:
            role_arn = operator.extra_kwargs["role_arn"]

            if "role_session_duration" in operator.extra_kwargs:
                role_session_duration = int(
                    operator.extra_kwargs["role_session_duration"]
                )
            else:
                try:
                    role_session_duration = find_role_session_duration(role_arn)
                except Exception:
                    operator.log.exception(
                        "Something when wrong with finding the max role session duration."
                    )
                    role_session_duration = MAX_ROLE_DURATION_SECONDS

            operator.log.info("Assuming role: {}".format(role_arn))

            host_credentials_path = os.path.join(host_tmp_dir, ".aws", "credentials")
            container_credentials_path = os.path.join(
                operator.tmp_dir, ".aws", "credentials"
            )

            raw_credentials = get_credentials(
                context=context,
                role_arn=role_arn,
                role_session_duration=role_session_duration,
            )
            log_credentials(operator, raw_credentials)
            credentials = format_credentials_data(raw_credentials)
            write_credentials(
                credentials=credentials, credentials_path=host_credentials_path
            )

            operator.environment[
                "AWS_SHARED_CREDENTIALS_FILE"
            ] = container_credentials_path
        else:
            operator.log.info("Not assuming role")



import json

from airflow.plugins_manager import AirflowPlugin
from airflow_docker.operator import BaseDockerOperator
from airflow_docker.views import template_folder
from flask import Blueprint
from flask_admin import BaseView, expose
from flask_appbuilder import BaseView as AppBuilderBaseView


def render_extensions(self):
    extensions_classes = BaseDockerOperator._extensions

    extensions = [
        {"name": klass.__name__, "doc": getattr(klass, "__doc__")}
        for klass in extensions_classes
    ]
    return extensions


class ExtensionsView(BaseView):
    @expose("/")
    def config(self):
        rendered_extensions = render_extensions(self)
        return self.render("extensions.html", data=rendered_extensions)


class AppBuilderExtensionsView(AppBuilderBaseView):
    template_folder = template_folder

    @expose("/")
    def list(self):
        rendered_extensions = render_extensions(self)
        return self.render_template(
            "extensions.html", data=rendered_extensions, appbuilder=True
        )


view = ExtensionsView(category="Airflow Docker", name="Extensions")
blueprint = Blueprint("extensions", __name__, template_folder=template_folder)

appbuilder_view = AppBuilderExtensionsView()
appbuilder_package = {
    "name": "Extensions",
    "category": "Airflow Docker",
    "view": appbuilder_view,
}



from setuptools import find_packages, setup


def parse_requirements(filename):
    with open(filename) as f:
        lineiter = (line.strip() for line in f)
        return [
            line.replace(" \\", "").strip()
            for line in lineiter
            if (
                line
                and not line.startswith("#")
                and not line.startswith("-e")
                and not line.startswith("--")
            )
        ]


with open("README.md", "rb") as f:
    LONG_DESCRIPTION = f.read().decode("utf-8")


setup(
    name="airflow-docker",
    version="1.1.0",
    description="An opinionated implementation of exclusively "
    "using airflow DockerOperators for all Operators",
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    author="Hunter Senft-Grupp",
    author_email="huntcsg@gmail.com",
    url="https://github.com/huntcsg/airflow-docker",
    license="Apache License 2.0",
    keywords="airflow docker",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Console",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Topic :: System :: Monitoring",
    ],
    packages=find_packages("src"),
    package_dir={"": "src"},
    zip_safe=False,
    python_requires=">=3,!=3.0.*,!=3.1.*,!=3.2.*,!=3.3.*,!=3.4.*,!=3.5.*",
    install_requires=parse_requirements("deps/requirements.in"),
    extras_require={
        "testing": parse_requirements("deps/testing-requirements.in"),
        "docs": parse_requirements("deps/docs-requirements.in"),
        "linting": parse_requirements("deps/linting-requirements.in"),
        "dev": parse_requirements("deps/dev-requirements.in"),
    },
    entry_points={
        "airflow.plugins": [
            "airflow_docker = airflow_docker.plugin:AirflowDockerPlugin"
        ]
    },
    package_data={'airflow-docker': ['src/airflow_docker/views/templates/*.html']},
    include_package_data=True,
)
